package be.iminds.ilabt.jfed.restapi.debugdata;

import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.jfed.util.RFC3339Util;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * ServerResourceMethodDD
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonIgnoreProperties("@context")
public class ServerResourceMethodDD implements DebugData {
    private static final Logger LOG = LoggerFactory.getLogger(ServerResourceMethodDD.class);

    private final Timestamp start;
    private final Timestamp stop;
    private final String method;
    private final List<ClientApiCallDD> clientApiCalls;
    private final List<LogLineDD> logLines;

    @JsonCreator
    public ServerResourceMethodDD(@JsonProperty("start") Timestamp start,
                                  @JsonProperty("stop") Timestamp stop,
                                  @JsonProperty("method") String method,
                                  @JsonProperty("clientApiCalls") List<ClientApiCallDD> clientApiCalls,
                                  @JsonProperty("logLines") List<LogLineDD> logLines) {
        this.start = start;
        this.stop = stop;
        this.method = method;
        this.clientApiCalls = clientApiCalls == null ? null : Collections.unmodifiableList(new ArrayList<>(clientApiCalls));;
        this.logLines = logLines == null ? null : Collections.unmodifiableList(new ArrayList<>(logLines));;
    }

    @JsonProperty
    @JsonDeserialize(using = RFC3339Util.TimeStampJsonDeserializer.class)
    @JsonSerialize(using = RFC3339Util.TimeStampJsonSerializer.class)
    public Timestamp getStart() {
        return start;
    }

    @JsonProperty
    @JsonDeserialize(using = RFC3339Util.TimeStampJsonDeserializer.class)
    @JsonSerialize(using = RFC3339Util.TimeStampJsonSerializer.class)
    public Timestamp getStop() {
        return stop;
    }

    @JsonProperty
    public String getMethod() {
        return method;
    }

    @JsonProperty
    public List<ClientApiCallDD> getClientApiCalls() {
        return clientApiCalls;
    }

    @JsonProperty
    public List<LogLineDD> getLogLines() {
        return logLines;
    }


    @JsonProperty(value="@type")
    public String getClassName() {
        return getClass().getSimpleName();
    }

    @JsonProperty(value="@type")
    private void setClassName(String className) {
        if (!getClass().getSimpleName().equals(className))
            throw new IllegalArgumentException("@type must be "+getClass().getSimpleName());
    }

    @Override
    public String toString() {
        ObjectMapper mapper = new ObjectMapper();
        try {
            return mapper.writerWithDefaultPrettyPrinter().writeValueAsString(this);
        } catch (JsonProcessingException e) {
            LOG.error("Exception converting "+getClass().getSimpleName()+" to JSON", e);
            return "Exception converting "+ getClass().getSimpleName() +" to JSON: "+e.getMessage();
        }
    }
}
